"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SoftDeletable = void 0;
require("./soft-deletable-handler.subscriber");
const core_1 = require("@mikro-orm/core");
const soft_deletable_symbol_1 = require("./soft-deletable.symbol");
const soft_deletable_filter_constant_1 = require("./soft-deletable-filter.constant");
const soft_deletable_handler_subscriber_1 = require("./soft-deletable-handler.subscriber");
/**
 * Mark an entity type as soft-deletable.
 * @param type - Helper function for type inference.
 * @param field - Identifier field used to identify deleted entities.
 * @param value - Value to set to the identifier field in deletions.
 * @returns
 * @see SoftDeletableHandlerSubscriber
 */
const SoftDeletable = (type, field, value) => (type) => {
    const metadata = { field, value };
    Reflect.defineMetadata(soft_deletable_symbol_1.SOFT_DELETABLE, metadata, type);
    (0, core_1.Filter)({
        name: soft_deletable_filter_constant_1.SOFT_DELETABLE_FILTER,
        cond: { [field]: null },
        default: true,
    })(type);
};
exports.SoftDeletable = SoftDeletable;
soft_deletable_handler_subscriber_1.SoftDeletableHandlerSubscriber;
//# sourceMappingURL=soft-deletable.decorator.js.map